;
; GRENADE LOBBER 3: SONIC BOOM - by Dave Hassler, Nov. 2022
;
; an "animated" game for PAL-1/microKIM.
;
; This program uses Jim Butterfield's MUSIC BOX from "The First Book
; of KIM" as a subroutine.  For sound, pin PB7 on the PAL-1 (or microKIM,
; or KIM-1) expansion connector should be fed into an audio amplifier.
;
; for best results, use 80x24 terminal screen, serial emulator at 4800 bps,
; and terminal emulator cursor either blank or an underscore.
;
; Released "to the wild" under the MIT License and copyright 2022 by
; David Hassler.  Please feel free to augment, fix, change, and make a 
; derivative work - just give credit for the original version.
;
; As Liu Ganning says: "Happy Hacking!"  --  DHH
;


FIELD  = $1780
STOREA = $20
NMEPOS = $21
MISSBY = $22
GRNADE = $23
TARGET = $24
STOREX = $25
GOTHIT = $26
LOBSND = $27

TUNEAD = $068C

; ***** Fixed locations for MUSIC BOX *****

	WORK    = $E0
        LIMIT   = $E6
        VAL2    = $E9
        VAL1    = $EA
        TIMER   = $EB
        XSAV    = $EC
        SBD     = $1742
        PBDD    = $1743



*=$0200

; CLEAR "SCREEN"
	CLD		; BECAUSE IT'S NICE
	LDA #$FF
	STA $F2		; RESET STACK POINTER
	LDX #$19	; 25 LINES-WORTH OF CRLF
	JSR CLEAR

; PRINT TITLE/INSTR.

	LDX #$00
PRNT1	LDA TITLE,X
	BEQ T_TUNE
	JSR $1EA0
	INX
	JMP PRNT1

T_TUNE	JSR OPENER	; play opening tune

	LDX #$00
PRNT3	LDA INSTR,X
	BEQ SEEDST
	JSR $1EA0
	INX
	JMP PRNT3

SEEDST	LDA $1744	; from 1 u-sec timer in the RIOT
	BEQ SEEDST	; if #00, get another 'seed'
	STA R_SEED	; store it in main RND routine seed holder


	LDX #$0A	; 10 more CRLFs to set up for play
	JSR CLEAR

; PRINT 'HEADER' OF 'YARD MARKERS'
	
	CLC
	LDA #$00
PRTYRD	JSR $1E3B	; print the byte
	ADC #$10	; add 16 (but = $20, so...  :^)
	CMP #$60	; is it $60?
	BEQ HDR2	; yes, branch to print rest of header
	STA $20		; otherwise, put A somewhere safe
	LDX #$08
SPCOUT	JSR $1E9E	; call OUTSP
	DEX
	BNE SPCOUT	; not #00, loop
	LDA $20		; otherwise, reload A
	BNE PRTYRD	; will always branch to print new byte

; MORE HEADER TEXT - STATUS BAR
HDR2	LDX #$00
PRNT2	LDA HEDTXT,X
	BEQ ONECR	; out to print CRLF and finsh header
	JSR $1EA0
	INX
	JMP PRNT2

ONECR	LDX #$01
	JSR CLEAR


; POSITION THE ENEMY BETWEEN #29-59, SET INIT # GRENADES, DIST TO ENEMY

NME1ST	JSR RAND_8	; get a random number 00-FF in A
	AND #$3F	; mask to keep bottom 6 bits (<#64) 
	CMP #$14	; is A <#20?
	BCC NME1ST	; yes, get another rnd #
	CMP #$32	; is A >#50?
	BCS NME1ST	; yes, get another rnd #
	STA NMEPOS	; store the AC in 'Enemy Pos.' location
;	LDA #$30	; TEMP/TEST: PUT HIM AT #48
;	STA NMEPOS
	LDA #$03	; HEX #
	STA GRNADE	; number of grenades

; SET UP INITAL PLAYING FIELD, VARIABLES

	CLD
HDR3	LDX #$00
POKE3	LDA URBASE,X
	BEQ FILFLD	; done 'poking' in player base
	STA FIELD,X
	INX
	JMP POKE3

FILFLD	LDX #$05	; LOAD UP REST OF FIELD WITH 'GROUND'
FLD2	LDA #$5F	; load up a '_'  to make 'ground'
	STA FIELD,X
	INX
	CPX #$39	; is it #57? (end of field of play +4)
	BNE FLD2
	LDA #$7C
	STA $17B4	; put the pipe char into the field
	LDA #$00	
	STA FIELD,X	; TERMINATE WITH ZERO AT END OF LINE
			; fall through to main game loop






; *** M A I N   L O O P ***

GMLOOP	JSR PRTFLD	; print the current field
	JSR GETTRG	; get new target for lob, test, move enemy
	JMP GMLOOP
	BRK





; *** SUBROUTINES

; LOB GRENADE

LOBDOT	LDA #$00
	STA LOBSND
	LDX #$05	; LOOP COUNTER
LOBSTR	LDA #$2E	; LOAD A DOT
	STA FIELD,X
	STX STOREX	; store, because of PRTFLD loop
	JSR PRTFLD	; go print the field

; *** PLAY A SOUND HERE

	JSR PLYTHR	; play lobbing sound



	LDX STOREX	; restore X
	LDA #$5F	; load 'ground'
	STA FIELD,X	; restore at curr. loc
	LDA FIELD+1,X	; get what's in NEXT field loc
	CMP #$2A	; is it '*'?
	BNE BUMPX	; no, inx and see if target reached
	
	INX		; test to see if curr target is a prev trgt
	CPX TARGET
	BNE BUMPX
	DEX
	JMP GOFELD

EXTRAX	INX		; extra INX to skip over the '*'
BUMPX	INX
	CPX #$40	; test for runaway condx when TRGT is <#10
	BEQ GOFELD
	CPX TARGET	
	BNE LOBSTR	; fill with '~' until -1 target
	LDA #$2A
	STA FIELD,X	; store a '*' at target
GOFELD	JSR PRTFLD
	RTS


; PRINT THE FIELD

PRTFLD	LDA #$0D	; load a CR
	JSR $1EA0	; just in case....
	LDX #$00
LOOP5	LDA FIELD,X	; get data in a field loc.
	JSR $1EA0	; print it
	INX		; + 1
	CPX #$39	; is X #57?
	BNE LOOP5	; no, loop
	JSR SPACE3
	LDA GRNADE
	JSR $1E3B	; print # of grenades left
	JSR SPACE3	; prepare for player input
	RTS


; GET PLAYER INPUT AND TEST

GETTRG	JSR $1F9D	; call GETBYT
	STA TARGET
	LDA #$0D	; print a CR
	JSR $1EA0

; TEST - LOB GRENADE AND CHECK FOR RESULT

GUDTRG	JSR DEC2HX	; convert the input 'byte' to hex
	STX TARGET	; place the result in target
	LDA TARGET
	CMP #$06	; target inside player fort?
	BCS TEST2	; >=5 THEN NEXT TEST
	JMP SUICID	; yes, blew self up!
TEST2	CMP #$32	; threw past limit?
	BCC GUDLOB	; no, OK
	LDA #$32	; yes, put the throw at the limit
	STA TARGET

GUDLOB	JSR LOBDOT	; LAUNCH the grenade!
	JSR PLYBOM	; *** PLAY 'EXPLOSION' SOUND HERE

	LDA GRNADE
	SEC
	SBC #$01	; one less grenade
	STA GRNADE


GOTHIM	SEC		; DID PLAYER GET HIM?
	LDA TARGET
	SBC NMEPOS
	BNE PLUS1	; if <>00 then next test
	JMP WINNER	; it's a match - winner!
PLUS1	CMP #$01	; is enemy +1 away?
	BNE MINUS1	; no, next test
	JMP WINNER	; close enough!
MINUS1	CMP #$FF	; is enemy -1 away?
	BNE DELTA	; no, then goto short/long test
	JMP WINNER	; just like horseshoes and nuclear war!

DELTA	CLC
	CMP #$B0	; DETERMINE SHORT OR LONG LOB
	BCC BIGLOB	; if result of subtraction is over $80, 
	LDX #$00	; then the throw was SHORT (?)
	BEQ AD2FLD	; set offset X to start of MISTXT and branch
BIGLOB	LDX #$05
AD2FLD	LDY #$35	; *** WAS $3F WHEN 60-YD FIELD
LOOP7	LDA MISTXT,X	; get a char
	BEQ TEST3	; #00?  out
	STA FIELD,Y	; poke it into the field
	INX
	INY
	JMP LOOP7	; 


TEST3	LDA GRNADE	; see if out of grenades
	BNE MOVNME	; if not, move NME
	JMP LOSER	; but if no grenades.....


; MOVE THE ENEMY 1-6 YRDS, SEE IF < 5 YRDS AWAY, AND COMPLETE THE TURN

MOVNME	JSR RAND_8	; get a random number 00-FF in A
	AND #$07	; mask to keep bottom 3 bits (<#08) 
	BEQ MOVNME	; if 00, get another rnd #
	STA STOREA	; put the move distance in temp var
	LDA NMEPOS	; load orig enemy position
	SEC		
	SBC STOREA	; subtract move dist from orig. pos.
	STA NMEPOS	; store new pos. in 'Enemy Pos.' location
	CMP #$09
	BCS DONE5	; is enemy w/i 5 yrds of the player's bunker?
	JMP LOSER	; yes, ...
DONE5	RTS		; otherwise, RTS to finish the main loop

; =============================
; SUBROUTINES AND ENDINGS
; =============================

CLEAR	LDA #$0A
	JSR $1EA0
	LDA #$0D
	JSR $1EA0
	DEX
	BNE CLEAR
	RTS

SPACE3	LDX #$03
LOOP6	JSR $1E9E	; CALL OUTSP
	DEX
	BNE LOOP6
	RTS


WINNER	CLD
	LDX #$00
	LDY #$0E	
POKE4	LDA WINTXT,X	
	BEQ DONE2	; done 'poking' in message
	STA FIELD,Y	; start at #14 on field
	INX
	INY
	JMP POKE4
DONE2	JSR PRTFLD	; print the ending message on the field
	LDA #$53	; store loc of tune in MB
	STA TUNEAD
	JSR SOUND	; call MB and play it
	JSR RESET	; reset I/O
	JSR $1E2F	; call CRLF
	BRK
	

LOSER	CLD
	LDX #$00	
	LDY #$0E
POKE5	LDA LTEXT,X	
	BEQ DONE3	; done 'poking' in message
	STA FIELD,Y	; start at #19 on field
	INX
	INY
	JMP POKE5
DONE3	JSR PRTFLD	; print the ending message on the field
	LDA #$72	; store loc of tune in MB
	STA TUNEAD
	JSR SOUND	; call MB and play it
	JSR RESET	; reset I/O
	JSR $1E2F	; call CRLF
	BRK


SUICID	CLD
	LDX #$00
	LDY #$0E	
POKE6	LDA DEDTXT,X	
	BEQ DONE4	; done 'poking' in message
	STA FIELD,Y	; start at #19 on field
	INX
	INY
	JMP POKE6
DONE4	JSR PRTFLD	; print the ending message on the field
	LDA #$72	; store loc of tune in MB
	STA TUNEAD
	JSR SOUND	; call MB and play it
	JSR RESET	; reset I/O
	JSR $1E2F	; call CRLF
	BRK


; SIMPLE HEX <-> DEC CONVERSION BY JACK CLARKE
; From "COMPUTE!" magazine, Issue 10, Pg. 60, March 1981
;
; Only Hex $00-$63 and Decimal #00-#99 work with this method. [?]

DEC2HX	SED		; SET DEC MODE
	SEC		; CLEAR CARRY
	LDX #$00	; CLEAR X
	LDA TARGET	; 'DECIMAL' NUMBER TO CNVRT
SUB1	SBC #$01	; SUBTRACT 1 FROM AC
	INX		; ADD 1 TO X
	CMP #$00	; IS AC ZERO?
	BNE SUB1	; NO, BRANCH TO SUB1
			; was INX (NEEDS A +1 BUMP) [but doesn't]
	CLD		; RESULT IS IN X
	RTS

HX2DEC	SED		; SET DEC MODE
	CLC		; CLEAR CARRY
	LDA #$00	; CLEAR AC
	LDX MISSBY	; HEX NUM TO CNVRT
ADD1	ADC #$01	; ADD 1 TO AC
	DEX		; SUBTRACT 1 FROM X
	BNE ADD1	; IS X ZERO?, NO, BRANCH TO ADD1
	CLD		; RESULT IS IN AC
	RTS





	; SIMPLE 8-BIT PSEUDO-RANDOM NUMBER GENERATOR
	; This next part (for any 6502) by Lee Davison, 2002.
	; from https://philpem.me.uk/leeedavison/6502/prng/index.html

rand_8
	LDA	r_seed		; get seed [THIS PROG. FROM 1 uSEC TIMER]
	AND	#$B8		; mask non feedback bits
				; for maximal length run with 8 bits we need
				; taps at b7, b5, b4 and b3
	LDX	#$05		; bit count (shift top 5 bits)
	LDY	#$00		; clear feedback count
F_loop
	ASL	A		; shift bit into carry
	BCC	bitclr		; branch if bit = 0

	INY			; increment feedback count (b0 is XOR all the
				; shifted bits from A)
bitclr
	DEX			; decrement count
	BNE	F_loop		; loop if not all done

no_clr
	TYA			; copy feedback count
	LSR	A		; bit 0 into Cb
	LDA	r_seed		; get seed back
	ROL	A		; rotate carry into byte
	STA	r_seed		; save number as next seed
	RTS			; done
	
r_seed
	.byte	1		; prng seed byte (must not be zero)

	; PLAY DESCENDING 'THROW' SOUNDS DURING LOB
	; ON EVERY OTHER DOT MOVE

PLYTHR	LDA LOBSND	; get index for lob sounds
	CMP #$80
	BCC PLAYIT	; <$80 (bit 7=0) go play it
	EOR #$80	; clear bit 7 for next time
	STA LOBSND
	RTS
PLAYIT	STA TUNEAD	; put the desired note into Music Box
	JSR SOUND	; call MB
	JSR RESET
	INC LOBSND	; then bump counter by 2
	INC LOBSND
	LDA LOBSND
	EOR #$80	; set bit 7 for next time
	STA LOBSND	; and store it
	RTS

RESET	LDA #$FE	; reset all I/O
	STA $1740
	LDA #$00
	STA $1741
	LDA #$47
	STA $1742
	LDA #$3F
	STA $1743
	RTS

PLYBOM	LDA #$2E
	STA TUNEAD
	JSR SOUND
	JSR RESET
	RTS

OPENER	LDA #$38
	STA TUNEAD
	JSR SOUND
	JSR RESET
	RTS




TITLE
.BYTE "GRENADE LOBBER 3 - by Dave Hassler (80-col, 4800 bps, PB7 sound)"
.BYTE $0A $0D $00

INSTR
.BYTE "You have only 3 grenades. The hidden enemy advances 1-7 m/turn"
.BYTE $0A $0D
.BYTE "and may start near or down range. Enter your launch distance"
.BYTE $0A $0D
.BYTE "at 'Trgt'. +/-1 m is a hit. Status report each turn. Good luck!"
.BYTE $00

HEDTXT
.BYTE "|Miss Left Trgt"
.BYTE $00

URBASE
.BYTE "[_/_]"
.BYTE $00

WINTXT
.BYTE " TARGET ELIMINATED - VICTORY! "
.BYTE $00

LTEXT
.BYTE " MISS - NO GRENADES - DEFEAT! "
.BYTE $00

DEDTXT
.BYTE " YOU BLEW YOURSELF UP, IDIOT! "
.BYTE $00

MISTXT
.BYTE "SHRT"
.BYTE $00
.BYTE "LONG"
.BYTE $00

;
; ====================================
; MUSIC BOX, BY JIM BUTTERFIELD
; ====================================
;

; PROGRAM - MUSIC BOX

; INITIALIZE - RESET WORK PARAMETERS

SOUND:  LDX     #5
LP1:    LDA     INIT,X
        STA     WORK,X
        DEX
        BPL     LP1

; MAIN ROUTINE HERE - WORK NOT RESET

GO:     LDA    #$BF
        STA    PBDD             ; open output channel
        LDY    #0
        LDA    (WORK+4),Y       ; get next note
        INC    WORK+4
        CMP    #$FA             ; test for halt
        BNE    NEXT
        RTS                     ; (was BRK)
        NOP
        BEQ    GO               ; resume when GO pressed
NEXT:   BCC    NOTE             ; is it a note?
        SBC    #$FB             ; if not, decode instrument
        TAX                     ; and put it into x
        LDA    (WORK+4),Y       ; GET PARAMETER
        INC    WORK+4
        STA    WORK,X           ; STORE IN WORK TABLE
        BCS    GO               ; UNCONDITIONAL BRANCH

; SET UP FOR TIMING NOTE

NOTE:   LDX    WORK             ; TIMING
        STX    LIMIT+1
        LDX    WORK+1           ; LONG NOTE FACTOR
        TAY                     ; TEST ACCUM.
        BMI    OVER             ; LONG NOTE?
        LDX    #1               ; NOPE, SET SHORT NOTE
OVER:   STX    LIMIT            ; store length factor
        AND    #$7F             ; remove short/long flag
        STA    VAL2
        BEQ    HUSH             ; is it a pause
        STA    VAL1             ; no, set pitch
HUSH:   LDA    VAL2             ; get timing and
        AND    WORK+3           ; bypass if muted
        BEQ    ON
        INC    VAL1             ; else fade the
        DEC    VAL2             ; note
ON:     LDX    VAL2
        LDA    #$A7
        JSR    SOUND1
        BMI    GO
        LDX    VAL1
        LDA    #$27
        JSR    SOUND1
        BMI    GO
        BPL    HUSH

; SUBROUTINE TO SEND A BIT

SOUND1: LDY    WORK+2           ; octave flag
        STY    TIMER
        STX    XSAV
SLOOP:  CPX    #0
        BNE    CONT
        LDX    XSAV
        DEC    TIMER
        BNE    SLOOP
        BEQ    SEX
CONT:   STA    SBD
        DEX
        DEC    LIMIT+2
        BNE    SLOOP
        DEC    LIMIT+1
        BNE    SLOOP
        LDY    WORK
        STY    LIMIT+1
        DEC    LIMIT
        BNE    SLOOP
        LDA    #$FF
SEX:    RTS


; INITIAL CONSTANTS

INIT:  
.BYTE    $18, 2, 1
.BYTE    $00, <SONGS, >SONGS     ; modified to put songs after program rather than at $0000


* = $0700

SONGS:
      .BYTE    $00,$00

; =========================================
; THE TUNES: lob sounds, splat sound, title tune, win tune, lose tune
; (Apple I hex)
;
; 0700: 1D FA 1F FA 21 FA 23 FA
; : 25 FA 27 FA 2A FA 2D FA
; : 30 FA 32 FA 36 FA 39 FA
; : 3C FA 40 FA 44 FA 48 FA
; : 4D FA 52 FA 57 FA 5C FA
; : 62 FA 68 FA 6E FA

; 072C: FE FF FB 40
; : FC 04 FD 02 F5 FA
; 
; 0736: FE 00 FB 30 68 62 FC 03 D7 68
; : D7 68 FC 02 D7 C0 D7 68 62 D7 D7 CD C4 FC 04 C0
; : FA
;
; 0751: FE 00 FB 28 39 32 FC 06 AA FC 02 9F FC 03 A5
; : 23 FC 04 9F FC 03 9B 1F FC 02 9B A5 FC 06 9F FA

; 0770: FE 00 FB 30 FC 02 FD 02 E2 E2 62 FC 03 E2 FC 02
; : D2 57 FC 02 D7 62 E2 68 FC 04 E2 FA
